@@ -467,9 +467,11 @@ float buffer_readsample_interpolated(buffer_info_t* buf, int channel, float inde
 		/* this is quite expensive: */
 		if (channel == 1)
 			return inter4pol_float(buf->chunk_list[group]->readfc->L,
-					index,buffer_get_size(buf) / buf->samplerate_factor) * buf->volume_factor;
+					index,(double)buffer_get_size(buf) * (double)buf->samplerate_factor) 
+				* buf->volume_factor;
 		else 	return inter4pol_float(buf->chunk_list[group]->readfc->R,
-					index,buffer_get_size(buf) / buf->samplerate_factor) * buf->volume_factor;
+					index,(double)buffer_get_size(buf) * (double)buf->samplerate_factor) 
+				* buf->volume_factor;
 	}
 	return 0.;
 }
