@@ -345,9 +345,9 @@ chunk_group_t* chunkgroup_new(int gid, int priority){
 void chunkgroup_delete(chunk_group_t* group){
 	if (!group) return;
 
-	filechunk_delete(group->readfc);
-	filechunk_delete(group->nextfc);
-	filechunk_delete(group->prevfc);
-	filechunk_delete(group->wrapfc);
+	if (group->readfc) filechunk_delete(group->readfc);
+	if (group->nextfc) filechunk_delete(group->nextfc);
+	if (group->prevfc) filechunk_delete(group->prevfc);
+	if (group->wrapfc) filechunk_delete(group->wrapfc);
 	free(group);
 }
