@@ -47,6 +47,8 @@ typedef struct _thread_info{
       	volatile int 		can_process;
 	volatile int		inside;
 	volatile int		outside;
+	volatile long		inrounds;
+	volatile long		outrounds;
 	jack_info_t 		*jack_info;
 	pthread_t 		mix_thread_id;
 	volatile long 		xruns;
@@ -75,6 +77,14 @@ submix_t* submix_new (thread_info_t *info, int id, char* name){
 	memset(submix->rb_outR->buf, 0, submix->rb_outR->size);
 	memset(submix->rb_outP->buf, 0, submix->rb_outP->size);
 
+	/* try mlock */
+	jack_ringbuffer_mlock(submix->rb_inL);
+	jack_ringbuffer_mlock(submix->rb_inR);
+	jack_ringbuffer_mlock(submix->rb_inP);
+	jack_ringbuffer_mlock(submix->rb_outL);
+	jack_ringbuffer_mlock(submix->rb_outR);
+	jack_ringbuffer_mlock(submix->rb_outP);
+
 	submix->needs_input = 0;
 	submix->did_output = 0;
 	submix->needs_pointer = 0;
@@ -88,26 +98,30 @@ submix_t* submix_new (thread_info_t *info, int id, char* name){
 		snprintf(tmp,MAX_NAMESIZE,"%s_in_L",name);
 		submix->port_inL = jack_port_register (info->jack_info->client, 
 					tmp, JACK_DEFAULT_AUDIO_TYPE, JackPortIsInput, 0);
+		if (!submix->port_inL) fprintf (stderr,"could not create %s_in_L\n",name);
 		snprintf(tmp,MAX_NAMESIZE,"%s_in_R",name);
 		submix->port_inR = jack_port_register (info->jack_info->client, 
 					tmp, JACK_DEFAULT_AUDIO_TYPE, JackPortIsInput, 0);
-
+		if (!submix->port_inR) fprintf (stderr,"could not create %s_in_R\n",name);
 		snprintf(tmp,MAX_NAMESIZE,"%s_in_Position",name);
 		submix->port_inP = jack_port_register (info->jack_info->client, 
 					tmp, JACK_DEFAULT_AUDIO_TYPE, JackPortIsInput, 0);
-
+		if (!submix->port_inP) fprintf (stderr,"could not create %s_in_Position\n",name);
 		snprintf(tmp,MAX_NAMESIZE,"%s_out_Position",name);
 		submix->port_outP = jack_port_register (info->jack_info->client, 
 					tmp, JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
+		if (!submix->port_outP) fprintf (stderr,"could not create %s_out_L\n",name);
 		info->outside = 101;
 
 	}
 	snprintf(tmp,MAX_NAMESIZE,"%s_out_L",name);
 	submix->port_outL = jack_port_register (info->jack_info->client, 
 				tmp, JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
+	if (!submix->port_outL) fprintf (stderr,"could not create %s_out_L\n",name);
 	snprintf(tmp,MAX_NAMESIZE,"%s_out_R",name);
 	submix->port_outR = jack_port_register (info->jack_info->client, 
 				tmp, JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
+	if (!submix->port_outR) fprintf (stderr,"could not create %s_out_R\n",name);
 	free(tmp);
 
 	submix->id = id;
@@ -225,7 +239,7 @@ void jack_shutdown (void *arg){
 }
 
 void errfunc(const char *msg){
-/*      	printf ("jack error: %s\n",msg);*/
+      	printf ("jack error: %s\n",msg);
 }
 
 void freewheelfunc (int starting, void *arg){
@@ -238,6 +252,7 @@ int xrunfunc (void *arg){
 	info->outside = 401;
         info->xruns++;
 	info->outside = 402;
+	printf ("jack xrun\n");
 	return 0;
 }
 
@@ -260,6 +275,7 @@ int process (jack_nframes_t nframes, void *arg){
         
 	info->inside = -3;                                        
         if (pthread_mutex_trylock (&info->mix_thread_lock)){
+/*		printf ("lock not possible \n");*/
                 return 0;       
         }                               
                                 
@@ -268,14 +284,17 @@ int process (jack_nframes_t nframes, void *arg){
         if (info->freewheeling)         
                 while (info->can_process != 1); 
         else if (info->can_process == 0){
-		/* -1 for free wheeling 1 for new data, 2 for init*/
+		/* -1 for free wheeling, 1 for new data, 2 for init*/
+/*		printf ("srun in process\n");*/
 		info->xruns++;
                 return 0;
         }  else if (info->can_process != 1) {
+/*		printf ("xrun in process\n");*/
                 return 0;
         }
                
 	info->inside = -1; 
+	info->inrounds++;	
         info->last_nframes = nframes;
        
 
@@ -288,8 +307,13 @@ int process (jack_nframes_t nframes, void *arg){
                                         rb = info->submixes[i]->rb_outL;
                                         rh = (jack_default_audio_sample_t *) jack_port_get_buffer (port,nframes);
                                         if ((rs = jack_ringbuffer_read (rb,(void*)rh,info->sample_size * nframes))
-                                                < nframes * info->sample_size){ info->xruns++;}
-                                        /*printf("%s->outL, %d/%d/%d frames\n",jack_port_name (info->submixes[i]->port_outL),nframes,info->last_nframes,rs);*/
+                                                < nframes * info->sample_size){ info->xruns++;
+/*					printf ("xrun in %d_did_output (outside:%d, rs:%d != %d, rounds:%ld/%ld)\n",
+							i,info->outside, rs, (info->sample_size * nframes), 
+							info->inrounds, info->outrounds); */
+/*                                      printf("%s->outL, %d/%d/%d frames\n",jack_port_name (info->submixes[i]->port_outL),
+							nframes,info->last_nframes,rs);*/
+					}
 
                                         port = info->submixes[i]->port_outR;
                                         rb = info->submixes[i]->rb_outR;
@@ -336,7 +360,8 @@ int process (jack_nframes_t nframes, void *arg){
                         }
                 }
         }
-       	pthread_cond_signal (&info->data_ready);
+/*       	pthread_cond_signal (&info->data_ready);*/
+	pthread_cond_broadcast (&info->data_ready);
 	info->inside = 2;
        	pthread_mutex_unlock (&info->mix_thread_lock);
 	info->inside = 3;
@@ -345,6 +370,7 @@ int process (jack_nframes_t nframes, void *arg){
 
 static void* mix_thread(void *arg){
 	thread_info_t *info = (thread_info_t*)arg;
+	struct sched_param rtparam;
 	jack_nframes_t i,j = 0;
 	looper_data_t* data = NULL;
 	looper_data_t* data2 = NULL; 
@@ -362,15 +388,14 @@ static void* mix_thread(void *arg){
 	float* mainbufL = (float*) malloc(sizeof(float) * DEFAULT_RB_SIZE);
 	float* mainbufR = (float*) malloc(sizeof(float) * DEFAULT_RB_SIZE);
 	int cc = 0;
-	char clientname[64];
+	char *clientname = malloc(jack_client_name_size() * sizeof(char));
 
         jack_set_error_function ((void*)errfunc);
 
         while (!info->jack_info->client && (cc < 99)){
-		memset(&clientname,0,64);
+		memset(clientname,0,jack_client_name_size());
                 sprintf(clientname,"kluppe%d",++cc);
                 info->jack_info->client = jack_client_new (clientname);
-                info->jack_info->client = info->jack_info->client;
         }
 
         if (cc > 98){
@@ -396,10 +421,15 @@ static void* mix_thread(void *arg){
 	info->jack_info->samplerate = (long)jack_get_sample_rate(info->jack_info->client);
         info->can_process = 2;
 
+	/* try to reach higher priority */
+	memset (&rtparam, 0, sizeof (rtparam));
+        rtparam.sched_priority = /*sched_get_priority_max(SCHED_RR) - */10;
+	i = pthread_setschedparam(pthread_self(),SCHED_RR,&rtparam);
+	/* if (i) printf ("couldn't set priority\n");*/
+
         pthread_setcancelstate (PTHREAD_CANCEL_ENABLE, NULL);
         pthread_setcanceltype (PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
 
-
 	pthread_mutex_lock (&info->mix_thread_lock);
 /*	printf ("mixer thread:%d\n",(int)getpid());*/
 
@@ -437,6 +467,9 @@ static void* mix_thread(void *arg){
 				}
 			}
 
+/* xruns even if all the processing is switched off:
+#if 0
+*/
 			info->outside = 304;
 
 			memset (mainbufL,0,info->rb_size/*last_nframes*/ * info->sample_size);
@@ -475,8 +508,8 @@ static void* mix_thread(void *arg){
 						printf ("can't find physical input ports for looper\n");
 					}
 					submix_append (info,submix,data->id);
-                                        free(name);
-                                        info->outside = 307;
+                                       	free(name);
+                                       	info->outside = 307;
 				}
 				info->outside = 309;
 				
@@ -497,9 +530,9 @@ static void* mix_thread(void *arg){
 						else	submix->needs_pointer = 0;
 	
 						/* LATER calculate the number of samples from 
-			   			jacks buffer_size_callback */
+		   				jacks buffer_size_callback */
 						/* for now, it's just the last number of samples
-			   			from process() */
+		   				from process() */
 						i = j;
 						j = info->last_nframes;
 						if (jack_ringbuffer_read_space(submix->rb_inL) > j * info->sample_size){
@@ -519,14 +552,14 @@ static void* mix_thread(void *arg){
 						}
 
 						oldrecpos = looperdata_get_recpos(data);
-                                                looperdata_lock(data);
-                                                info->outside = 311;
+                                               	looperdata_lock(data);
+                                               	info->outside = 311;
 
 						if (submix->needs_input){
 							jack_ringbuffer_read (submix->rb_inL,
 								(void *) Lbuf, info->sample_size * j);
 							jack_ringbuffer_read (submix->rb_inR,
-                                                                (void *) Rbuf, info->sample_size * j);
+                                                               	(void *) Rbuf, info->sample_size * j);
 /*							looperdata_write_sampleblock(data, j, Lbuf, Rbuf);*/
 						}							
 						if (submix->needs_pointer){
@@ -536,27 +569,33 @@ static void* mix_thread(void *arg){
 					
 						submix->did_output = 0;	
 
-						looperdata_calc_sampleblock(data, j, Lbuf, Rbuf, Pbuf);
+						looperdata_calc_sampleblock(data, j, Lbuf, Rbuf, Pbuf,
+							!info->jack_info->suspend_playing, 
+							!info->jack_info->suspend_recording);
 
-						if (data->isplaying){	
+						if (data->isplaying){
+							looperdata_unlock (data);	
 							if (submix->did_pointer)
-                                                        	jack_ringbuffer_write (submix->rb_outP,
-                                                                	(void *)Pbuf, info->sample_size * j);
+                                                       		jack_ringbuffer_write (submix->rb_outP,
+                                                               		(void *)Pbuf, info->sample_size * j);
 							noutports++;
-							for (i = 0; i < j;i++){
-                                                                *(mainbufL + i) += *(Lbuf + i);
-                                                                *(mainbufR + i) += *(Rbuf + i);         
-                                                        }
-							if (jack_port_connected(submix->port_outL) ||
-                                                        	jack_port_connected(submix->port_outR)){	
-								jack_ringbuffer_write (submix->rb_outL,
-									(void *)Lbuf, info->sample_size * j);
-								jack_ringbuffer_write (submix->rb_outR,
-                                                                	(void *)Rbuf, info->sample_size * j);
-								submix->did_output = 1;
+							if (!info->jack_info->suspend_playing)
+								for (i = 0; i < j;i++){
+                                                               		*(mainbufL + i) += *(Lbuf + i);
+                                                               		*(mainbufR + i) += *(Rbuf + i);         
+                                                       		}
+							if ((submix->port_outL) && (submix->port_outR)){
+								if (jack_port_connected(submix->port_outL) ||
+                                                       			jack_port_connected(submix->port_outR)){	
+									jack_ringbuffer_write (submix->rb_outL,
+										(void *)Lbuf, info->sample_size * j);
+									jack_ringbuffer_write (submix->rb_outR,
+                                                               			(void *)Rbuf, info->sample_size * j);
+									submix->did_output = 1;
+								}
 							}
-						}
-						looperdata_unlock (data);
+						} else looperdata_unlock (data);
+
 						info->outside = 312;
 /*						info->last_nframes = j;*/
 
@@ -605,9 +644,8 @@ static void* mix_thread(void *arg){
 			speciallist_unlock(info->looperdatalist);
 /*			printf ("unlock:%d\n",count);*/
 			/* LATER delete unused submixes */
-
 			info->outside = 314;
-			if (noutports > 1){
+			if ((noutports > 1) && (!info->jack_info->suspend_playing)){
 				/* normalize main out signal */
 				mixfactor = 1.0f / sqrt(noutports);
 				for (i = 0; i < j/*info->last_nframes*/; i++){
@@ -622,14 +660,19 @@ static void* mix_thread(void *arg){
 				info->jack_info->xruns = info->xruns;
 /*				info->xruns = 0;*/
 			}
+/* end of xrun test 
+#endif
+*/
 			info->outside = 316;
 			/* write main out data */
+/*			printf ("write:%d bytes\n",info->last_nframes * info->sample_size);*/
 			jack_ringbuffer_write (info->submixes[0]->rb_outL,(void *)mainbufL,info->last_nframes * info->sample_size);
 		       	jack_ringbuffer_write (info->submixes[0]->rb_outR,(void *)mainbufR,info->last_nframes * info->sample_size);	
 			info->outside = 317;
 			info->can_process = 1;
 		}/* end if can_process*/
 		/* wait until process() signals more data */
+		++info->outrounds;
 		info->outside = 318;
 		pthread_cond_wait (&info->data_ready, &info->mix_thread_lock);
 		info->outside = 319;
@@ -657,6 +700,8 @@ jack_info_t* mixer_launch (speciallist_t *looperdatalist){
 	info->freewheeling = 0;
 	info->inside = 0;
 	info->outside = 0;
+	info->inrounds = 0;
+	info->outrounds = 0;
 	info->sample_size = sizeof(jack_default_audio_sample_t);
 	info->jack_info = (jack_info_t*) malloc(sizeof(jack_info_t));
         info->jack_info->client = 0;
@@ -665,6 +710,8 @@ jack_info_t* mixer_launch (speciallist_t *looperdatalist){
         info->jack_info->xruns = 0;
         info->jack_info->name = NULL;
         info->jack_info->freewheeling = 0;
+	info->jack_info->suspend_playing = 0;
+	 info->jack_info->suspend_recording = 0;
 	info->jack_info->alive = 0;
 
 
@@ -673,25 +720,13 @@ jack_info_t* mixer_launch (speciallist_t *looperdatalist){
 	info->rb_size = DEFAULT_RB_SIZE;
 
 	/* launch mix thread */
-/*
-        pthread_attr_init(&attr);
-        policy = SCHED_FIFO;
-	result = pthread_attr_setinheritsched(&attr, PTHREAD_EXPLICIT_SCHED);
-	if (result) printf ("setinherit failed\n");
-	result = pthread_attr_setscope(&attr, PTHREAD_SCOPE_SYSTEM);
-	if (result) printf ("setscope failed\n");
-	result = pthread_attr_setschedpolicy(&attr, policy);
-	if (result) printf ("setsched failed\n");
-*/
 	pthread_cond_init(&info->data_ready, NULL);
 	pthread_mutex_init(&info->mix_thread_lock,NULL);
-/*	pthread_create (&info->mix_thread_id, &attr, mix_thread, info);*/
 	pthread_create (&info->mix_thread_id, NULL, mix_thread, info);
 
 	while (!info->can_process && (++waited < 300)){ 
 		usleep(10);
 	}
-		
 
 	if (info->can_process < 1) return info->jack_info;
 
