@@ -70,6 +70,13 @@ looper_data_t* looperdata_new(buffer_info_t* buf, long samplerate,int id){
 	data->limit = 1.;
 	data->limknee = .5;
 	data->limconst = .25;
+	data->thresholdstate = THRESHOLDSTATE_ANY;
+	data->low_threshold = .01;
+	data->high_threshold = .015;
+	data->thresholdmode = 0;
+	data->threshringptr = 0;
+	data->threshringsize = 22050; /* half a second at regular speed should be a good start */
+	data->threshring = malloc (sizeof(float) * data->threshringsize);
 	data->playvuL = .0;
 	data->playvuR = .0;
 	data->recvuL = .0;
@@ -198,7 +205,7 @@ void looperdata_copy_settings (looper_data_t* dst, looper_data_t* src){
 			dst->loopstart = src->loopstart;
 			if (dst->loopstart > buffer_get_size(dst->buf)){
 				dst->loopstart = 0; 
-				/* LATER find a mor intelligent solution */
+				/* LATER find a more intelligent solution */
 			}
 			dst->loopend = src->loopend;	
 			if (dst->loopend > buffer_get_size(dst->buf)){
@@ -207,7 +214,7 @@ void looperdata_copy_settings (looper_data_t* dst, looper_data_t* src){
 			dst->recstart = src->recstart;
 			if (dst->recstart > buffer_get_size(dst->buf)){
 				dst->recstart = 0;
-				/* LATER find a mor intelligent solution */
+				/* LATER find a more intelligent solution */
 			}
 			dst->recend = src->recend;
 			if (dst->recend > buffer_get_size(dst->buf)){
@@ -305,14 +312,16 @@ void looperdata_write_dirtyblock(looper_data_t* data, double start, double end){
 	block = speciallist_get_first(data->dirtylist);
 
 	while (block){
+/*
 		if (((block->start <= start) && (block->end >= start)) ||
 			((block->start <= end) && (block->end >= end)) ){
-			if (!overlap){
-				/* LATER: delete if previous overlap overlaps */
-				overlap = 1;
+*/
+		if (((block->start <= (start + 1.)) && (block->end >= start)) ||
+                        ((block->start <= end) && (block->end >= (end - 1.))) ){
 				if (block->start > start) block->start = start;
 				if (block->end < end) block->end = end;
-			}
+				overlap = 1;
+				break;
 		}
 		block = speciallist_get_next(data->dirtylist,(void*)block);
 	}
@@ -875,6 +884,53 @@ void looperdata_set_limit(looper_data_t* data, float limit){
 }
 
 
+int looperdata_get_thresholdmode(looper_data_t* data){
+	return data->thresholdmode;
+}
+
+void   looperdata_set_thresholdmode(looper_data_t* data, int mode){
+	if (mode){ 
+		data->thresholdmode = 1;	
+		data->thresholdstate = THRESHOLDSTATE_LOW;
+	} else {
+		data->thresholdmode = 0;
+		data->thresholdstate = THRESHOLDSTATE_ANY;
+	}
+}
+
+double looperdata_get_low_threshold(looper_data_t* data){ 
+	return data->low_threshold;;
+}
+
+void   looperdata_set_low_threshold(looper_data_t* data, double low){
+	if (low >= 0.)
+		data->low_threshold = low;
+}
+
+double looperdata_get_high_threshold(looper_data_t* data){
+	return data->high_threshold;	
+}
+
+void   looperdata_set_high_threshold(looper_data_t* data, double high){
+	if (high >= 0.)
+		data->high_threshold = high;
+}
+
+int    looperdata_get_thresholdbuffersize(looper_data_t* data){
+	return data->threshringsize;
+}
+
+void   looperdata_set_thresholdbuffersize(looper_data_t* data, int size){
+	if ((size > 64) && (size < 882000)){ /* could be more sophisticated */
+		data->threshringsize = size;
+		if (data->threshringptr > data->threshringsize)
+			data->threshringptr = 0;
+		data->threshring = (float*) realloc (data->threshring, 
+					sizeof(float) * data->threshringsize);
+	}
+}
+
+
 /*
 void looperdata_set_grainlength (looper_data_t* data, long length){
 	if (length < 256){length = 256;}
@@ -1341,7 +1397,7 @@ void looperdata_calc_sample_mono(looper_data_t* data, float* L){
 	}
 }
 
-void looperdata_calc_sampleblock (looper_data_t* data, long size, float* Lbuf, float* Rbuf, float* Pbuf){
+void looperdata_calc_sampleblock (looper_data_t* data, long size, float* Lbuf, float* Rbuf, float* Pbuf, int do_play, int do_rec){
 	long l;
 	long rp;
 	double pp;
@@ -1353,10 +1409,11 @@ void looperdata_calc_sampleblock (looper_data_t* data, long size, float* Lbuf, f
 	if ((size <= (data->recend - data->recstart) || !data->isrecording) && 
 		0 && /* this part is broken, use the single sample copy mode instead */
 		(buffer_get_sampleratefactor(data->buf, data->samplerate) == 1.0)){
-		if (data->isrecording){ /* same result for l++ mode */
+		/* LATER: include rec-threshold here when reactivating blockwise operation */
+		if (data->isrecording && do_rec){ /* same result for l++ mode */
 			looperdata_write_sampleblock(data, size, Lbuf, Rbuf);
 		}
-		if (data->isplaying){
+		if (data->isplaying && do_play){
 			if (buffer_get_channels(data->buf) > 1){
 	        		if (data->panswing){
 					for (l = 0; (l < size) && (data->isplaying); l++){
@@ -1401,15 +1458,38 @@ void looperdata_calc_sampleblock (looper_data_t* data, long size, float* Lbuf, f
 			} /* end if stereo/mono */
 		}
 	} else { /* else have to work sample by sample */
+		if (data->thresholdmode){ /* check recording thresholds: */
+			double max = 0.;
+			/* write new data to ringbuffer first:*/
+			for (l = 0; l < size ; l++){
+				if (buffer_get_channels(data->buf) > 1){
+					if (fabs(*(Lbuf + l)) > fabs(*(Rbuf + l)))
+						*(data->threshring + data->threshringptr) = fabs(*(Lbuf + l));
+					else	*(data->threshring + data->threshringptr) = fabs(*(Rbuf + l));
+				} else 	*(data->threshring + data->threshringptr) = fabs(*(Lbuf + l));
+				if (++data->threshringptr >= data->threshringsize) data->threshringptr = 0;
+			}
+			/* check ringbuffer for maximum: */
+			for (l = 0; l < data->threshringsize ; l++){
+                                if (*(data->threshring + l) > max) max = *(data->threshring + l);
+                        }
+
+			if (max < data->low_threshold){
+				data->thresholdstate = THRESHOLDSTATE_LOW;
+			} else  if (max > data->high_threshold) {
+				data->thresholdstate = THRESHOLDSTATE_HIGH;
+			}
+		}
 		if ((buffer_get_sampleratefactor(data->buf, data->samplerate) == 1.0) && 
-			data->isrecording && !data->isplaying ){	/* copy at once anyway */
+			data->isrecording && !data->isplaying  && do_rec && data->thresholdstate){	
+			/* copy at once anyway */
 			looperdata_write_sampleblock(data, size, Lbuf, Rbuf);	
-		}else {
+		} else {
 			if (buffer_get_channels(data->buf) > 1){	
 				for (l = 0; l < size ; l++){
-					if (data->isrecording)
+					if (data->isrecording && do_rec && data->thresholdstate)
 						looperdata_write_sample (data, (Lbuf + l), (Rbuf + l));
-					if (data->isplaying){
+					if (data->isplaying && do_play){
 						if (data->panswing){
 							data->pointer = *(Pbuf + l);
                                                 	looperdata_calc_sample_stereo (data, (Lbuf + l), (Rbuf + l));
@@ -1431,9 +1511,9 @@ void looperdata_calc_sampleblock (looper_data_t* data, long size, float* Lbuf, f
 				}
 			} else { /* mono: */
 				for (l = 0; l < size; l++){
-					if (data->isrecording)
+					if (data->isrecording && do_rec && data->thresholdstate)
 						looperdata_write_sample (data, (Lbuf + l), NULL);
-					if (data->isplaying){
+					if (data->isplaying && do_play){
                                                 if (data->panswing){
                                         		data->pointer = *(Pbuf + l);
                                         		looperdata_calc_sample_mono (data, (Lbuf + l));
@@ -1458,7 +1538,7 @@ void looperdata_calc_sampleblock (looper_data_t* data, long size, float* Lbuf, f
 		} /* end if playing */
 	} /* end if size < loopsize */
 
-	if (data->isplaying){
+	if (data->isplaying && do_play){
 		/* reverb?: */
                 if (data->reverblength > 0){
                         for (l = 0;l < size;l++){
@@ -1481,6 +1561,7 @@ void looperdata_calc_sampleblock (looper_data_t* data, long size, float* Lbuf, f
                                         *(Rbuf + l) = -data->limit - data->limconst / *(Rbuf + l);
                         }
                 }/* end limiter */
+
                 for (l = 0;l < size;l++){
                         if (data->outmaxL < *(Lbuf + l)){data->outmaxL = *(Lbuf + l);}
                         if (data->outmaxR < *(Rbuf + l)){data->outmaxR = *(Rbuf + l);}
