@@ -42,7 +42,7 @@ snd_seq_t *open_seq(const char* name) {
 
   	if (snd_seq_open(&seq_handle, "hw", SND_SEQ_OPEN_DUPLEX, 0) < 0) {
     		fprintf(stderr, "Error opening ALSA sequencer.\n");
-    		exit(1);
+    		return NULL; // exit(1);
   	}
 
   	snd_seq_set_client_name(seq_handle, name);
@@ -50,7 +50,7 @@ snd_seq_t *open_seq(const char* name) {
             	SND_SEQ_PORT_CAP_WRITE|SND_SEQ_PORT_CAP_SUBS_WRITE,
             	SND_SEQ_PORT_TYPE_APPLICATION)) < 0) {
     		fprintf(stderr, "Error creating sequencer port.\n");
-    		exit(1);
+    		return NULL; // exit(1);
   	}
   	return(seq_handle);
 }
@@ -120,7 +120,7 @@ void midi_action(midi_info_t *minfo) {
 				while(ld){
 					int mc = looperdata_get_midichannel (ld);
 					if ((mc < 0) || (mc == ev->data.control.channel))
-						looperdata_set_grainpitchtablevalue (ld, note, 0.);
+/*						looperdata_set_grainpitchtablevalue (ld, note, 0.);*/
 					ld = speciallist_get_next_nolock (minfo->looperdatalist, ld);
 				}
 				break;
@@ -145,6 +145,7 @@ static void* midi_run (void * data) {
   	signal (SIGHUP, SIG_IGN);
 
   	minfo->seq = open_seq("kluppe");
+	if (!minfo->seq) return NULL;
 	minfo->seq_nfds = snd_seq_poll_descriptors_count(minfo->seq, POLLIN);
 	minfo->pfds = (struct pollfd*)alloca(minfo->seq_nfds * sizeof(struct pollfd));
 	snd_seq_poll_descriptors(minfo->seq, minfo->pfds, minfo->seq_nfds, POLLIN);
