@@ -31,6 +31,7 @@
 
 
 #define DIR_MODE (S_IRWXU|S_IRGRP)
+#define DEFAULT_IDLETIME 80
 
 /* global widgets: */
 GtkWidget *globalbox;
@@ -38,6 +39,8 @@ GtkWidget *footertable;
 GtkWidget *footerbox;
 GtkWidget *newlooperbutton;
 GtkWidget *newbufferbutton;
+GtkWidget *suspend_playingbutton;
+GtkWidget *suspend_recordingbutton;
 GtkWidget *newemptybufferbutton;
 GtkWidget *savebutton;
 GtkWidget *openbutton;
@@ -241,10 +244,13 @@ void get_load (){
 			uptime = ut + utk * .01;
 			idle = id + idk * .01;
 			lastload = 1.0 - (float)(idle - lastidle) / (float)(uptime - lastuptime) ;
+/*			printf ("up:%.2lf, %.2lf, idle:%.2lf, %.2lf, lastload:%.2lf ",
+				uptime,lastuptime,idle,lastidle,lastload);*/
 			lastuptime = uptime;
 			lastidle = idle;	
 /*			if (!(0.0 < lastload <= 1.0)) lastload = 0.0;*/
 			if (!((lastload > 0.0) && ((lastload < 1.0) || (lastload == 1.0)))) lastload = 0.0;
+/*			printf ("lastload after:%.2lf\n",lastload);*/
 		}
 		fclose(ufile);	
 	}
@@ -393,7 +399,6 @@ gint main_idlefunc(gpointer data){
 
 		loadfilestate++;
 	}else if (!not_ready && (loadfilestate == 2)){
-		gint x,y,w,h;
 		while ((ti = config_next_looper(ch)) != 0){
 /*			printf ("new looper:%d\n",ti);*/
 			if (looperindexcounter < ti) looperindexcounter = ti;
@@ -406,13 +411,11 @@ gint main_idlefunc(gpointer data){
 			gtk_box_pack_start (GTK_BOX (globalbox), looper->mainbox, FALSE, FALSE, 0);
 			looper_update_bufferlist(looper,buffer_list);
 		}
-		config_get_geometry(ch, &x,&y,&w,&h);
-                gtk_window_resize((GtkWindow*)window,w,h);
-                gtk_window_move((GtkWindow*)window,x,y);
 /*		printf ("stage 2\n");*/
 		loadfilestate++;
 		countdown = 5; /* wait a while to settle widgets */
 	}else if (loadfilestate == 3){
+		gint x,y,w,h;
 /*		printf ("stage 3\n");*/
 
 		tmpptr = g_list_first (looperlist->listptr);
@@ -423,8 +426,12 @@ gint main_idlefunc(gpointer data){
 					config_read_looper (ch,looper);
 			tmpptr = g_list_next(tmpptr);
 		}
+		config_get_geometry(ch, &x,&y,&w,&h);
+                gtk_window_move((GtkWindow*)window,x,y);
+                gtk_window_resize((GtkWindow*)window,w,h);
+/*                printf ("w:%d, h:%d, x:%d, y:%d\n",w,h,x,y);*/
 		gtk_widget_destroy (waitdialog);
-		loadfilestate = 0;
+                loadfilestate = 0;
 		set_status("project loaded");
 
 		close_config_reader(ch);
@@ -436,10 +443,10 @@ gint main_idlefunc(gpointer data){
 
 		/* change idle scheduling to relax at high loads*/
 		get_load();
+/*		timeout version: */
 		if (idle_id) g_source_remove (idle_id);
         	if (lastload > .4) idle_id =  g_timeout_add((guint)(lastload * 100),(GSourceFunc)main_idlefunc,NULL);
-        	else idle_id =  g_timeout_add(40,(GSourceFunc)main_idlefunc,NULL);
-
+        	else idle_id =  g_timeout_add(DEFAULT_IDLETIME,(GSourceFunc)main_idlefunc,NULL);
 
 		txt = realloc(txt,sizeof(char) * 256);
 		memset (txt,0,255);
@@ -558,6 +565,23 @@ static void open_discstream(gpointer data){
         g_free (filename);
 }
 
+static void suspend_playing(gpointer data){
+	GtkWidget* suspend_playingbutton = (GtkWidget*)data;
+
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(suspend_playingbutton)))
+		jack_info->suspend_playing = 1;
+	else 	jack_info->suspend_playing = 0;
+}
+
+static void suspend_recording(gpointer data){
+        GtkWidget* suspend_recordingbutton = (GtkWidget*)data;
+
+        if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(suspend_recordingbutton)))
+                jack_info->suspend_recording = 1;
+        else    jack_info->suspend_recording = 0;
+}
+
+
 static void open_project (char *filename){
 	char* pathend = NULL;
 	struct stat     fstat;
@@ -719,7 +743,7 @@ static void save_project_as_menucallback(gpointer data){
 			/* gtk_window_get_position () to save position too ??? */
 			gtk_window_get_size(GTK_WINDOW (window),&w,&h);
 			gtk_window_get_position(GTK_WINDOW (window),&x,&y);
-			ret = save_config (looperlist->listptr, buffer_list, lastpath, lastconfigfilepath, filename,x,y,w,h);
+			ret = save_config (looperlist->listptr, buffer_list, lastpath, lastconfigfilepath, filename,x,y,w,h,jack_info->samplerate);
 			if (ret < 0)	set_status("saving failed !");
 			else {
 				projectfilename = realloc (projectfilename, sizeof(char) * (strlen(filename) +1));
@@ -749,7 +773,7 @@ static void save_project_menucallback(gpointer data){
 	gtk_window_get_size(GTK_WINDOW (window),&w,&h);
 	gtk_window_get_position(GTK_WINDOW (window),&x,&y);
 	if (projectfilename)
-		ret = save_config (looperlist->listptr, buffer_list, lastpath, lastconfigfilepath, projectfilename,x,y,w,h);
+		ret = save_config (looperlist->listptr, buffer_list, lastpath, lastconfigfilepath, projectfilename,x,y,w,h,jack_info->samplerate);
 	else save_project_as_menucallback(data);
 
 	if (ret < 0)  set_status("saving failed !");
@@ -865,6 +889,16 @@ static void create_mainlayout(){
 	gtk_box_pack_start (GTK_BOX (footerbox),newemptybufferbutton,FALSE, FALSE, 0);
 	gtk_widget_show (newemptybufferbutton);
 
+        suspend_playingbutton = gtk_toggle_button_new_with_label (" suspend playback ");
+        g_signal_connect (G_OBJECT(suspend_playingbutton), "toggled", G_CALLBACK(suspend_playing), NULL);
+        gtk_box_pack_start (GTK_BOX (footerbox),suspend_playingbutton,FALSE, FALSE, 0);
+        gtk_widget_show (suspend_playingbutton);
+
+	suspend_recordingbutton = gtk_toggle_button_new_with_label (" suspend rec. ");
+        g_signal_connect (G_OBJECT(suspend_recordingbutton), "toggled", G_CALLBACK(suspend_recording), NULL);
+        gtk_box_pack_start (GTK_BOX (footerbox),suspend_recordingbutton,FALSE, FALSE, 0);
+        gtk_widget_show (suspend_recordingbutton);
+
 	status = gtk_statusbar_new();
 	gtk_statusbar_set_has_resize_grip((GtkStatusbar*)status, TRUE);
 	gtk_box_pack_start (GTK_BOX (footerbox),status, TRUE, TRUE, 0);
@@ -1010,21 +1044,10 @@ int main(int argc, char *argv[])
 
 	defaultdir = g_get_home_dir ();
 	defaultdir = g_strconcat (defaultdir,"/.kluppe",NULL);
-	
-/*	pthread_setcanceltype (PTHREAD_CANCEL_ASYNCHRONOUS, NULL);*/
-
-	g_thread_init(NULL);
-	gdk_threads_init();
-	gdk_threads_enter();
-
-	gtk_init(&argc, &argv);
-
+		
 	/* global data initialisation */
-/*	bufferlistmutex = g_mutex_new ();*/
 	buffer_list = speciallist_new();
 
-	looperlist = looper_list_new();
-
 	looperdatalist = speciallist_new();
 
 	/* start the loadbuffer thread */
@@ -1033,7 +1056,6 @@ int main(int argc, char *argv[])
 #ifdef HAVE_ALSA
         /* create midi thread: */
         midi_info = midi_info_new(looperdatalist);
-/*        printf ("midi thread-create done\n"); */
 #endif
 
 #ifdef HAVE_OSC
@@ -1043,12 +1065,20 @@ int main(int argc, char *argv[])
 	/* start the jack threads */
         jack_info = mixer_launch(looperdatalist);
 
+        g_thread_init(NULL);
+        gdk_threads_init();
+/*        gdk_threads_enter();*/
+
+	gtk_init(&argc, &argv);
+
+        looperlist = looper_list_new();
+
 	/* create main gui elements */
 	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	gtk_window_set_title (GTK_WINDOW (window), jack_info->name);
 	g_signal_connect(window, "destroy",G_CALLBACK(destroy), NULL);
 	g_signal_connect (G_OBJECT (window), "delete_event",G_CALLBACK (delete_event), NULL);
-	/*gtk_container_set_border_width(GTK_CONTAINER (window), 10);*/
+	/* gtk_container_set_border_width(GTK_CONTAINER (window), 10);*/
 
 	create_mainlayout();
 	
@@ -1072,9 +1102,11 @@ int main(int argc, char *argv[])
 	clickmode_list = clickmodes_create(jack_info->samplerate);
 
 	/* idle callback looks for changes in buffers and loopers */		
-/*	gtk_timeout_add(40,(GtkFunction)main_idlefunc,NULL);*/
-	idle_id = g_timeout_add(40,(GSourceFunc)main_idlefunc,NULL);
-/*	gtk_idle_add (main_idlefunc,NULL);*/
+	/* timeout version: */
+	idle_id = g_timeout_add(DEFAULT_IDLETIME,(GSourceFunc)main_idlefunc,NULL);
+
+	/* idle version (eats a lot of cpu */
+/*	gtk_idle_add_priority(G_PRIORITY_LOW,main_idlefunc,NULL);*/
 
 	retval = mkdir(defaultdir, DIR_MODE);
 	if (retval)
@@ -1089,7 +1121,7 @@ int main(int argc, char *argv[])
 	if (argc > 1) open_project(argv[1]);
 
 	gtk_main();
-	gdk_threads_leave();
+/*	gdk_threads_leave();*/
 
 	mixer_close(jack_info);
 
